/*

    Meta-Environment - An environment for language prototyping.
    Copyright (C) 2000  Stichting Mathematisch Centrum, Amsterdam,
                        The Netherlands.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA

*/
/*
  $Id: path.c,v 1.2 2000/05/12 12:18:21 olivierp Exp $  
 */

/*
 * The version of se is written by Mark van den Brand.
 */
             

#include <stdlib.h>

#include "se.h"   
#include "path.h"

ATerm newPath()
{
  ATermInt i = ATmakeInt(0);

  return ATmake("path(<list>)", ATmakeList1((ATerm) i));
}

ATerm goRight(ATerm path)
{
  ATermList pathElements;
  ATerm lastElement;
  int number;

  if(ATmatch(path, "path(<list>)", &pathElements)) {
    lastElement = ATgetLast(pathElements);
    pathElements = ATgetPrefix(pathElements);

    number = ATgetInt((ATermInt)lastElement);
    number++;
    lastElement = (ATerm)ATmakeInt(number);

    pathElements = ATappend(pathElements, lastElement);
    return ATmake("path(<list>)", pathElements); 
  }
  else {
    ATerror("Illegal path %t\n", path);
    return NULL; /* Silence the compiler */
  } 
}

ATerm goLeft(ATerm path)
{
  ATermList pathElements;
  ATerm lastElement;
  int number;

  if(ATmatch(path, "path(<list>)", &pathElements)) {
    lastElement = ATgetFirst(pathElements);
    pathElements = ATgetPrefix(pathElements);

    number = ATgetInt((ATermInt)lastElement);
    if(number > 0) {
      number--;
    }
    lastElement = (ATerm)ATmakeInt(number);
    pathElements = ATappend(pathElements, lastElement);

    return ATmake("path(<list>)", pathElements); 
  }
  else {
    ATerror("Illegal path %t\n", path);
    return NULL; /* Silence the compiler */
  } 
}

ATerm goDown(ATerm path) 
{
  ATermList pathElements;
  ATerm newElement, newPath;

  if(ATmatch(path, "path(<list>)", &pathElements)) {
    newElement = (ATerm)ATmakeInt(0);

    pathElements = ATappend(pathElements, newElement);
    newPath = ATmake("path(<list>)", pathElements);
    return newPath;
  }
  else {
    ATerror("Illegal path %t\n", path);
    return NULL; /* Silence the compiler */
  }
}      

ATerm goUp(ATerm path)
{
  ATermList pathElements;

  if(ATmatch(path, "path(<list>)", &pathElements)) {
    if(ATgetLength(pathElements) > 1) {
      pathElements = ATgetPrefix(pathElements);
    }
    return ATmake("path(<list>)", pathElements); 
  }
  else {
    ATerror("Illegal path %t\n", path);
    return NULL; /* Silence the compiler */
  } 
}

ATerm getPathInTree(ATerm tree, int i, ATerm path)
{
  int termLength, leftLength;
  ATerm termTerm, leftWs;

  if(ATgetType(tree) == AT_LIST) {
    ATermList treeList = (ATermList) tree;  
    ATerm tree1 = ATgetFirst(treeList);
    int j;
    j = getTreeLength(tree1);
    if(i>j) {
      treeList = ATgetNext(treeList);
      return getPathInTree((ATerm) treeList, i-j, goRight(path));
    }
    else {
      if(asfix_is_appl(tree1)) {
        return getPathInTree((ATerm)ATgetArgument((ATermAppl)tree1, 2), 
                             i, goDown(path));
      }
      else if(asfix_is_list(tree1)) {
        return getPathInTree((ATerm)ATgetArgument((ATermAppl)tree1, 2), 
                             i, goDown(path));
      }
      else if(asfix_is_lex(tree1) || asfix_is_var(tree1)) {
        return path;
      }
      else if(asfix_is_term(tree1)) {
        leftWs = ATgetArgument(tree1, 5);
        leftLength = getTreeLength(leftWs);
        if(i > leftLength) {
          i = i - leftLength;
          termTerm = ATgetArgument(tree1, 6);
          termLength = getTreeLength(termTerm);
          if(i > termLength) {                  
            return path;
          }
          else {
            return getPathInTree(termTerm, i, goRight(goDown(path)));
          }
        }
      }
      else {
        return goUp(path);
      }
    }
  }
  else if(ATgetType(tree) == AT_APPL) {
    if(asfix_is_appl(tree)) {
      return getPathInTree((ATerm)ATgetArgument((ATermAppl)tree, 2), 
                           i, goDown(path));
    }
    else if(asfix_is_list(tree)) {
      return getPathInTree((ATerm)ATgetArgument((ATermAppl)tree, 2), 
                           i, goDown(path));
    }
    else if(asfix_is_term(tree)) {
      leftWs = ATgetArgument(tree, 5);
      leftLength = getTreeLength(leftWs);
      if(i > leftLength) {
        i = i - leftLength;
        termTerm = ATgetArgument(tree, 6);
        termLength = getTreeLength(termTerm);
        if(i > termLength) {                  
          return path;
        }
        else {
          return getPathInTree(termTerm, i, goRight(goDown(path)));
        }
      }
      else {
        return path;
      }
    }
    else {
      return path;
    }
  }
  else {
    ATerror("In getPathInTree: Tree type not supported\n");
    return path;
  }
}

ATbool isLeafPath(ATerm path) 
{
  ATermList pathElements;
  ATerm firstElement;
  int number;

  if(ATmatch(path, "path(<list>)", &pathElements)) {
    if(ATgetLength(pathElements) == 1) {
      firstElement = ATgetFirst(pathElements);
      number = ATgetInt((ATermInt)firstElement);
      if(number == 0) {
        return ATtrue;
      }
      else {
        return ATfalse;
      }
    }
    else {
      return ATfalse;
    }
  }
  else {
    ATerror("Illegal path %t\n", path);
    return ATfalse;
  }
}      

ATbool isHeadZero(ATerm path) 
{
  ATermList pathElements;
  ATerm firstElement;
  int number;

  if(ATmatch(path, "path(<list>)", &pathElements)) {
    if(ATgetLength(pathElements) >= 1) {
      firstElement = ATgetFirst(pathElements);
      number = ATgetInt((ATermInt)firstElement);
      if(number == 0) {
        return ATtrue;
      }
      else {
        return ATfalse;
      }
    }
    else {
      return ATfalse;
    }
  }
  else {
    ATerror("Illegal path %t\n", path);
    return ATfalse;
  }
}      

ATerm getTail(ATerm path) 
{
  ATerm newPath;
  ATermList pathElements;

  if(ATmatch(path, "path(<list>)", &pathElements)) {
    newPath = ATmake("path(<list>)", ATgetNext(pathElements));
    return newPath;
  }
  else {
    ATerror("Illegal path %t\n", path);
    return path;
  }
}      

ATerm decreaseHead(ATerm path) 
{
  ATerm newPath;
  ATermList pathElements;
  ATerm firstElement;
  int number;

  if(ATmatch(path, "path(<list>)", &pathElements)) {
    firstElement = ATgetFirst(pathElements);
    pathElements = ATgetNext(pathElements);

    number = ATgetInt((ATermInt)firstElement);
    if(number > 0) {
      number--;
    }
    firstElement = (ATerm)ATmakeInt(number);

    newPath = ATmake("path(<list>)", ATinsert(pathElements, firstElement));
    return newPath;
  }
  else {
    ATerror("Illegal path %t\n", path);
    return path;
  }
}

ATbool isChildPath(ATerm pathOne, ATerm pathTwo)
{
  ATermList pathOneElements, pathTwoElements;

  if(ATmatch(pathOne, "path(<list>)", &pathOneElements)) {
    if(ATmatch(pathTwo, "path(<list>)", &pathTwoElements)) {
      if(!ATisEmpty(pathOneElements) && 
         !ATisEmpty(pathTwoElements)) {
        ATerm headPathOne = ATgetFirst(pathOneElements);
        ATerm headPathTwo = ATgetFirst(pathTwoElements);
        if(ATisEqual(headPathOne, headPathTwo)) {
          ATerm newPathOne = ATmake("path(<list>)", ATgetNext(pathOneElements));
          ATerm newPathTwo = ATmake("path(<list>)", ATgetNext(pathTwoElements));
          return isChildPath(newPathOne, newPathTwo);
        }
        else {
          return ATfalse;
        }
      }
      else {
        return ATtrue;
      }
    }
    else {
      ATerror("Illegal path %t\n", pathTwo);
      return ATfalse; 
    }
  }
  else {
    ATerror("Illegal path %t\n", pathOne);
    return ATfalse; 
  }
}
